/**
 * Copyright 2012 52North Initiative for Geospatial Open Source Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.n52.android.tracker.activities;

import org.n52.android.tracker.R;
import org.n52.android.tracker.Utils;
import org.n52.android.tracker.services.impl.TrackerService;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.location.Location;
import android.os.Bundle;
import android.os.Parcelable;
import android.widget.TextView;

/**
 * The Data activity provides the actual location values measured by the location provider. These location data will be send to the SOS service.
 * 
 * @author Jan Schulte
 */
public class Data extends Activity {

	private TextView longitude;
	private TextView latitude;
	private TextView altitude;
	private TextView accuracy;
	private TextView provider;
	private TextView speed;
	private TextView time;

	// holds the lastest measured location
	public static Location lastLoc;
	
	private BroadcastReceiver broadcastReceiver = new BroadcastReceiver() {
		@Override
		public void onReceive(Context context, Intent intent) {
			setLocation(intent);
		}
	};

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.data);
		longitude = (TextView) findViewById(R.id.data_longitude);
		latitude = (TextView) findViewById(R.id.data_latitude);
		altitude = (TextView) findViewById(R.id.data_altitude);
		accuracy = (TextView) findViewById(R.id.data_accuracy);
		provider = (TextView) findViewById(R.id.data_provider);
		speed = (TextView) findViewById(R.id.data_speed);
		time = (TextView) findViewById(R.id.data_time);
		setLocation(Data.lastLoc);
	}

	private void setLocation(Intent intent) {
		Parcelable location = intent.getParcelableExtra("location");
		if (location instanceof Location) {
			setLocation((Location) location);
		}
	}

	private void setLocation(Location location) {
		if (location != null) {
			longitude.setText(" " + location.getLongitude());
			latitude.setText(" " + location.getLatitude());
			altitude.setText(" " + location.getAltitude() + " m");
			accuracy.setText(" " + location.getAccuracy() + " m");
			provider.setText(" " + location.getProvider());
			speed.setText(" " + location.getSpeed() * 3.6 + " km/h");
			time.setText(Utils.getStringTime(location.getTime()));
		}
	}

	@Override
	protected void onResume() {
		super.onResume();
		registerReceiver(broadcastReceiver, new IntentFilter(TrackerService.ACTION_LOCATION));
	}

	@Override
	protected void onPause() {
		super.onPause();
		unregisterReceiver(broadcastReceiver);
	}

}
