/**
 * Copyright 2012 52North Initiative for Geospatial Open Source Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.n52.android.tracker.activities;

import org.n52.android.tracker.R;
import org.n52.android.tracker.Utils;

import android.app.Activity;
import android.location.Location;
import android.os.Bundle;
import android.text.format.Time;
import android.view.View;
import android.widget.TextView;

/**
 * The Logging activity provides log states, which can be fired by Logging.addEntry("LOG IT!").
 * 
 * @author Jan Schulte
 *
 */
public class Logging extends Activity {

	private static StringBuffer logging = new StringBuffer();

	private static TextView loggingView;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.logging);
		loggingView = (TextView) findViewById(R.id.logging_field);
		loggingView.setText(logging);
	}

	public void onClickClear(final View button) {
		logging.setLength(0);
		loggingView.setText(logging);
	}

	public static void addLocation(Location location) {
		StringBuffer temp = new StringBuffer();
		temp.append(Utils.round(location.getLongitude(), 4) + " ");
		temp.append(Utils.round(location.getLatitude(), 4) + " ");
		temp.append(" Acc: " + location.getAccuracy());
		addEntry(temp.toString());
	}

	@Override
	protected void onResume() {
		loggingView.setText(logging);
		super.onResume();
	}

	public static void addEntry(String log) {
		Time timeFormat = new Time();
		timeFormat.set(System.currentTimeMillis());
		logging.append(timeFormat.format("%H:%M:%S "));
		logging.append(log + "\n");
		if (loggingView != null) {
			try {
				loggingView.setText(logging);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
}
