/**
 * Copyright 2012 52North Initiative for Geospatial Open Source Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * 
 */
package org.n52.android.tracker.activities;

import java.util.List;

import org.n52.android.tracker.Configuration;
import org.n52.android.tracker.R;
import org.n52.android.tracker.connection.IConnection;
import org.n52.android.tracker.connection.InsertResultConnection;
import org.n52.android.tracker.connection.SensorBusConnection;
import org.n52.android.tracker.connection.TSOSConnection;
import org.n52.android.tracker.datastructure.Phenomenon;
import org.n52.android.tracker.datastructure.Point;
import org.n52.android.tracker.datastructure.Sensor;
import org.n52.android.tracker.services.ITrackerServiceBinder;
import org.n52.android.tracker.services.impl.TrackerService;
import org.n52.android.tracker.services.impl.TrackerService.TrackerServiceBinder;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import android.text.format.DateFormat;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

/**
 * In the StartRecord activity the Tracker mechanism can be started.
 * 
 * @author Jan Schulte
 *
 */
public class StartRecord extends Activity {

	private ITrackerServiceBinder trackerServBinder;

	private TextView txtName;
	private EditText editName;
	private TextView txtLabel;
	private EditText editLabel;
	private TextView txtDistance;
	private EditText editDistance;
	private TextView txtTime;
	private EditText editTime;
	private Button startStopButton;

	private ServiceConnection servConn = new ServiceConnection() {

		public void onServiceDisconnected(ComponentName name) {
			Logging.addEntry("Service Disconneted");
		}

		public void onServiceConnected(ComponentName name, IBinder binder) {
			trackerServBinder = (TrackerServiceBinder) binder;
		}
	};

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		Log.i("StartRecord", "onCreate()");
		super.onCreate(savedInstanceState);
		setContentView(R.layout.start_record);

		txtName = (TextView) findViewById(R.id.start_record_txt_name);
		editName = (EditText) findViewById(R.id.start_record_edit_name);
		txtLabel = (TextView) findViewById(R.id.start_record_txt_label);
		editLabel = (EditText) findViewById(R.id.start_record_edit_label);
		txtDistance = (TextView) findViewById(R.id.start_record_txt_distanz);
		editDistance = (EditText) findViewById(R.id.start_record_edit_distanz);
		txtTime = (TextView) findViewById(R.id.start_record_txt_time);
		editTime = (EditText) findViewById(R.id.start_record_edit_time);
		startStopButton = (Button) findViewById(R.id.start_record_button_start_stop);

		// set button text
		if (!TrackerService.tracking) {
			createStartView();
		} else {
			createStopView();
		}

	}

	private void createStartView() {
		startStopButton.setText(R.string.start_record_button_start);
		txtName.setVisibility(TextView.VISIBLE);
		editName.setVisibility(EditText.VISIBLE);
		txtLabel.setVisibility(TextView.VISIBLE);
		editLabel.setVisibility(EditText.VISIBLE);
		txtDistance.setVisibility(TextView.VISIBLE);
		editDistance.setVisibility(EditText.VISIBLE);
		txtTime.setVisibility(TextView.VISIBLE);
		editTime.setVisibility(EditText.VISIBLE);

		editDistance.setText(Preferences.getPreferences(this).getString(this.getString(R.string.prefs_key_minmeters),
				this.getString(R.string.prefs_default_minmeters)));
		editTime.setText(Preferences.getPreferences(this).getString(this.getString(R.string.prefs_key_minseconds),
				this.getString(R.string.prefs_default_minseconds)));

		// initial TrackName
		editName.setText(DateFormat.format("MM/dd/yy hh:mm", System.currentTimeMillis()));
	}

	private void createStopView() {
		startStopButton.setText(R.string.start_record_button_stop);
		txtName.setVisibility(TextView.GONE);
		editName.setVisibility(EditText.GONE);
		txtLabel.setVisibility(TextView.GONE);
		editLabel.setVisibility(EditText.GONE);
		txtDistance.setVisibility(TextView.GONE);
		editDistance.setVisibility(EditText.GONE);
		txtTime.setVisibility(TextView.GONE);
		editTime.setVisibility(EditText.GONE);
	}

	@Override
	protected void onStart() {
		Log.i("StartRecord", "onStart()");
		super.onStart();
	}

	@Override
	protected void onPause() {
		Log.i("StartRecord", "onPause()");
		super.onPause();
		unbindService(servConn);
	}

	@Override
	protected void onResume() {
		Log.i("StartRecord", "onResume()");
		// connect to service
		Intent trackerIntent = new Intent(this, TrackerService.class);
		bindService(trackerIntent, servConn, Context.BIND_AUTO_CREATE);
		super.onResume();
	}

	public void startStopTrack(View view) {
		if (!TrackerService.tracking) {
			final EditText name = (EditText) findViewById(R.id.start_record_edit_name);
			final EditText label = (EditText) findViewById(R.id.start_record_edit_label);
			// TODO put in configuration
			String procedureID = "procedure@" + name.getText();
			String startPoint = "startpoint";
			String gmlID = "user";
			Point point = new Point(0.0, 0.0, 0.0);
			if (Data.lastLoc != null) {
				point.setLongitude(Data.lastLoc.getLongitude());
				point.setLatitude(Data.lastLoc.getLatitude());
				point.setAltitude(Data.lastLoc.getAltitude());
			}
			List<Phenomenon> phenomenonList = Configuration.getPhenomenonList();
			String offering = name.getText().toString();
			String offeringDesc = label.getText().toString();
			Sensor sensor = new Sensor(procedureID, startPoint, point, gmlID, phenomenonList, offering, offeringDesc);
			float minDistance = Float.valueOf(editDistance.getText().toString());
			long minTime = Integer.valueOf(editTime.getText().toString());
			String string = this.getString(R.string.prefs_key_connectiontype);
			String connectionString = Preferences.getPreferences(this).getString(string, "transactional");
			IConnection connection = null;
			if (connectionString.equals("transactional")) {
				connection = new TSOSConnection(Preferences.getPreferences(this)
						.getString(this.getString(R.string.prefs_key_sosurl),
								this.getString(R.string.prefs_default_sosurl)));
			} else if (connectionString.equals("sensorBus")) {
				connection = new SensorBusConnection(Configuration.SENSOR_BUS_SERVER,
						Configuration.SENSOR_BUS_LOGIN,
						Configuration.SENSOR_BUS_PASSWORD,
						Configuration.SENSOR_BUS_CHANNEL);
			} else if (connectionString.equals("insertResult")) {
				connection = new InsertResultConnection(Preferences.getPreferences(this)
						.getString(this.getString(R.string.prefs_key_sosurl),
								this.getString(R.string.prefs_default_sosurl)));
			}
			if (connection != null) {
				createStopView();
				trackerServBinder.startTrack(sensor, minTime, minDistance, connection);
			} else {
				Logging.addEntry("Error: No connection type set!");
			}
		} else {
			createStartView();
			trackerServBinder.stopTrack();
		}
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		menu.add(0, Menu.FIRST, Menu.NONE, "Evaluation");
		return super.onCreateOptionsMenu(menu);
	}
	
}
