/**
 * Copyright 2012 52North Initiative for Geospatial Open Source Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * 
 */
package org.n52.android.tracker.connection;

import java.io.IOException;
import java.io.UnsupportedEncodingException;

import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.n52.android.tracker.Utils;
import org.n52.android.tracker.activities.Logging;
import org.n52.android.tracker.datastructure.Observation;
import org.n52.android.tracker.datastructure.Sensor;
import org.n52.android.tracker.encoder.RequestBuilder;

import android.util.Log;

/**
 * @author Jan Schulte
 * 
 */
public class InsertResultConnection implements IConnection {

	private static final String TAG = "InsertResultConnection";

	private String templateID;

	private String sosUrl;

	public InsertResultConnection(String sosUrl) {
		this.sosUrl = sosUrl;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.schulte.connection.IConnection#registerSensor(de.schulte.datastructure
	 * .Sensor)
	 */
	public void registerSensor(Sensor sensor) {
		Log.d(TAG, "registerSensor: entered");

		sensor.setOfferingDescription(sensor.getOfferingDescription() + " by InsertResult");

		String registerSensor = RequestBuilder.createRegisterSensor(sensor);
		String response = sendPostRequest(registerSensor);
		if (response == null) {
			// TODO No response - error occurred while send request
			return;
		}
		if (response.contains("<sos:AssignedSensorId>")) {
			Logging.addEntry("Add Sensor: "
					+ response.substring(response.indexOf("<sos:AssignedSensorId>") + 22,
							response.indexOf("</sos:AssignedSensorId>")));
		} else {
			Logging.addEntry(response);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.schulte.connection.IConnection#insertObservation(de.schulte.datastructure
	 * .Sensor, de.schulte.datastructure.Observation)
	 */
	public boolean insertObservation(Observation observation) {
		Log.d(TAG, "insertObservation: entered");
		// first-time when no templateID exists, create a full insertObservation
		if (templateID == null) {
			String insertObservation = RequestBuilder.createInsertObservation(observation, true);
			String response = sendPostRequest(insertObservation);
			if (response == null) {
				return false;
			}
			if (response.contains("<sos:AssignedObservationId>")) {
				templateID = response.substring(response.indexOf("<sos:AssignedObservationId>") + 27,
						response.indexOf("</sos:AssignedObservationId>"));
				Logging.addEntry("Add ObservationTemplate: " + templateID);
				return true;
			} else if (response.contains("<ows:ExceptionText>")) {
				Logging.addEntry("Add Obs Error: "
						+ response.substring(response.indexOf("<ows:ExceptionText>") + 19,
								response.indexOf("</ows:ExceptionText>")));
				return true;
			} else {
				Logging.addEntry(response);
			}
		} else {
			// templateID exists, then use insertResult requests
			String insertResult = RequestBuilder.createInsertResult(observation, templateID);
			String response = sendPostRequest(insertResult);
			if (response.contains("<sos:AssignedObservationId>")) {
				Logging.addEntry("Add Observation by InsertResult: "
						+ response.substring(response.indexOf("<sos:AssignedObservationId>") + 27,
								response.indexOf("</sos:AssignedObservationId>")));
				return true;
			} else if (response.contains("<ows:ExceptionText>")) {
				Logging.addEntry("Add InsertResult Error: "
						+ response.substring(response.indexOf("<ows:ExceptionText>") + 19,
								response.indexOf("</ows:ExceptionText>")));
				return true;
			} else {
				Logging.addEntry(response);
			}
		}
		return false;
	}

	public String getConnectionType() {
		return "InsertResult";
	}

	private String sendPostRequest(String request) {
		HttpClient httpClient = new DefaultHttpClient();
		HttpPost httpPost = new HttpPost(sosUrl);
		String response = null;
		try {
			StringEntity entity = new StringEntity(request);
			httpPost.setEntity(entity);
			response = Utils.convertStreamToString(httpClient.execute(httpPost).getEntity().getContent());
		} catch (UnsupportedEncodingException e) {
			Log.e(TAG, "UnsupportedEncodingException", e);
		} catch (ClientProtocolException e) {
			Log.e(TAG, "ClientProtocolException", e);
		} catch (IOException e) {
			Log.e(TAG, "IOException", e);
		}
		return response;
	}

	public void stop() {
		// do nothing
	}
}
