/**
 * Copyright 2012 52North Initiative for Geospatial Open Source Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.n52.android.tracker.encoder;

import java.io.IOException;
import java.io.InputStream;

import org.n52.android.tracker.R;
import org.n52.android.tracker.Utils;

import android.content.Context;
import android.util.Log;

/**
 * Imports the template snippets to build transactional request to the SOS service
 * 
 * @author Jan Schulte
 * 
 */
public class Templates {

	private static String registerSensor;

	private static String insertObservation;

	private static String insertResult;

	private static String singleSensor;

	private static String input;

	private static String outputBoolean;

	private static String outputCount;

	private static String outputQuantity;

	private static String outputText;

	private static String outputPosition;

	private static String component;

	private static String fieldBoolean;

	private static String fieldCount;

	private static String fieldQuantity;

	private static String fieldText;

	/**
	 * Einladen der Request Templates
	 * 
	 * @param ctx
	 */
	public static void initTemplates(Context ctx) {
		try {
			// init registerSensor
			InputStream is = ctx.getResources().openRawResource(R.raw.reg_sens);
			registerSensor = Utils.convertStreamToString(is);
			// init insertObservation
			is = ctx.getResources().openRawResource(R.raw.insert_observation);
			insertObservation = Utils.convertStreamToString(is);
			// init insertResult
			is = ctx.getResources().openRawResource(R.raw.insert_result);
			insertResult = Utils.convertStreamToString(is);
			// init singleSensor
			is = ctx.getResources().openRawResource(R.raw.single_sensor);
			singleSensor = Utils.convertStreamToString(is);
			// init input
			is = ctx.getResources().openRawResource(R.raw.input);
			input = (Utils.convertStreamToString(is));
			// init output boolean
			is = ctx.getResources().openRawResource(R.raw.output_boolean);
			outputBoolean = (Utils.convertStreamToString(is));
			// init output count
			is = ctx.getResources().openRawResource(R.raw.output_count);
			outputCount = Utils.convertStreamToString(is);
			// init output quantity
			is = ctx.getResources().openRawResource(R.raw.output_quantity);
			outputQuantity = Utils.convertStreamToString(is);
			// init output text
			is = ctx.getResources().openRawResource(R.raw.output_text);
			outputText = Utils.convertStreamToString(is);
			// init output position
			is = ctx.getResources().openRawResource(R.raw.output_position);
			outputPosition = Utils.convertStreamToString(is);
			// init component
			is = ctx.getResources().openRawResource(R.raw.component);
			component = Utils.convertStreamToString(is);
			// init field boolean
			is = ctx.getResources().openRawResource(R.raw.field_boolean);
			fieldBoolean = Utils.convertStreamToString(is);
			// init field count
			is = ctx.getResources().openRawResource(R.raw.field_count);
			fieldCount = Utils.convertStreamToString(is);
			// init field quantity
			is = ctx.getResources().openRawResource(R.raw.field_quantity);
			fieldQuantity = Utils.convertStreamToString(is);
			// init field text
			is = ctx.getResources().openRawResource(R.raw.field_text);
			fieldText = Utils.convertStreamToString(is);
		} catch (IOException e) {
			Log.e("Tracker", e.getMessage());
		}
	}

	public static String getFieldBoolean() {
		return fieldBoolean;
	}

	public static String getFieldCount() {
		return fieldCount;
	}

	public static String getFieldQuantity() {
		return fieldQuantity;
	}

	public static String getFieldText() {
		return fieldText;
	}

	/**
	 * @return RegisterSensor Template
	 */
	public static String getRegisterSensorTemplate() {
		if (registerSensor == null) {
			Log.e("Tracker", "RegisterSensor template not initialized");
		}
		return registerSensor;
	}

	/**
	 * @return InsertObservation Template
	 */
	public static String getInsertObservationTemplate() {
		if (insertObservation == null) {
			Log.e("Tracker", "InsertObservation template not initialized");
		}
		return insertObservation;
	}

	/**
	 * @return InsertResult Template
	 */
	public static String getInsertResult() {
		return insertResult;
	}

	public static String getInput() {
		return input;
	}

	public static String getOutputBoolean() {
		return outputBoolean;
	}

	public static String getOutputCount() {
		return outputCount;
	}

	public static String getOutputQuantity() {
		return outputQuantity;
	}

	public static String getOutputText() {
		return outputText;
	}

	public static String getOutputPosition() {
		return outputPosition;
	}

	public static String getComponent() {
		return component;
	}

	public static String getSingleSensor() {
		return singleSensor;
	}
}
