/**
 * Copyright 2012 52North Initiative for Geospatial Open Source Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.n52.android.tracker.sensorbus;

public class ConnectSensorMessage extends AbstractSensorBusMessage {

    public final static String CONNECT_SENSOR_MSG = "ConnectSensor";

    private String sensorDescriptionURL;

    /**
     * @param sensorID
     * @param sensorDescriptionURL
     */
    public ConnectSensorMessage(String sensorDescriptionURL) {
        this.sensorDescriptionURL = sensorDescriptionURL;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.n52.sensorbus.msg.AbstractSensorBusMessage#createMessage()
     */
    public String createMessage() {
        StringBuffer sb = new StringBuffer();

        sb.append(CONNECT_SENSOR_MSG);
        sb.append(getSeparator());
        sb.append(this.sensorDescriptionURL);

        return sb.toString();
    }

    /**
     * 
     * @param messageString
     * @return
     */
    public static ConnectSensorMessage parseMessage(String messageString) {
        String[] msg = messageString.split(getParsingSeparator());

        if ( !msg[0].equals(CONNECT_SENSOR_MSG) || msg.length != 2) {
            throw new IllegalArgumentException("The message '" + messageString
                    + "' does not conform to the ConnectSensorMessage format.");
        }

        return new ConnectSensorMessage(msg[1]);
    }

    /**
     * @return the sensorDescriptionURL
     */
    public String getSensorDescriptionURL() {
        return this.sensorDescriptionURL;
    }
}
