/**
 * Copyright 2012 52North Initiative for Geospatial Open Source Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.n52.android.tracker.sensorbus;

public class PublishDataMessage extends AbstractSensorBusMessage {

    public final static String PUBLISH_DATA_MSG = "PublishData";

    /**
     * Unique Identifier for each sensor
     */
    private String descriptionURL;

    /**
     * actual time
     */
    private String timeTag;

    /**
     * data value
     */
    private String data;

    /**
     * observedProperty
     */
    private String observedProperty;

    /**
     * location (for mobile sensors)
     */
    private String locationTag;

    /**
     * 
     * @param sensorID
     * @param timeStamp
     * @param observedProperty
     * @param locationTag
     * @param dataValue
     */
    public PublishDataMessage(String descriptionURL,
                              String timeStamp,
                              String locationTag,
                              String observedProperty,
                              String dataValue) {
        this.descriptionURL = descriptionURL;
        this.timeTag = timeStamp;
        this.data = dataValue;
        this.observedProperty = observedProperty;
        this.locationTag = locationTag;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.n52.sensorbus.msg.AbstractSensorBusMessage#createMessage()
     */
    public String createMessage() {
        StringBuffer sb = new StringBuffer();

        sb.append(PUBLISH_DATA_MSG);
        sb.append(getSeparator());
        sb.append(this.descriptionURL);
        sb.append(getSeparator());
        sb.append(this.timeTag);
        sb.append(getSeparator());
        sb.append(this.observedProperty);
        sb.append(getSeparator());
        sb.append(this.locationTag);
        sb.append(getSeparator());
        sb.append(this.data);

        return sb.toString();
    }

    /**
     * Parses the message.
     * 
     * @param messageString
     *        the message string
     * @return the data message
     */
    public static PublishDataMessage parseMessage(String messageString) {
        String[] splittedMsg = messageString.split(getParsingSeparator());

        if (!splittedMsg[0].equals(PUBLISH_DATA_MSG) || splittedMsg.length != 6) {
            throw new IllegalArgumentException("The message '" + messageString
                    + "' does not conform to the PublishDataMessage format.");
        }

        return new PublishDataMessage(splittedMsg[1], splittedMsg[2], splittedMsg[3], splittedMsg[4], splittedMsg[5]);
    }

    /**
     * Gets the sensor id.
     * 
     * @return the sensor id
     */
    public String getSensorID() {
        return this.descriptionURL;
    }

    /**
     * Gets the time stamp.
     * 
     * @return the time stamp
     */
    public String getTimeTag() {
        return this.timeTag;
    }

    /**
     * Gets the data data.
     * 
     * @return the data data
     */
    public String getData() {
        return this.data;
    }

    /**
     * Gets the observed properties.
     * 
     * @return observed properties
     */
    public String getAdvertisedOutput() {
        return this.observedProperty;
    }

    public String getLocationTag() {
        return this.locationTag;
    }

}
