/**
 * Copyright 2012 52North Initiative for Geospatial Open Source Software GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * 
 */
package org.n52.android.tracker.services.impl;

import java.util.Stack;

import org.n52.android.tracker.activities.Logging;
import org.n52.android.tracker.connection.IConnection;
import org.n52.android.tracker.datastructure.Observation;
import org.n52.android.tracker.datastructure.Sensor;


/**
 * @author Jan Schulte
 * 
 */
public class RequestManagerThread extends Thread {

	private IConnection connection;

	private Sensor sensor;

	private Stack<Observation> obsStack;

	private boolean tracking;

	public RequestManagerThread(IConnection connection, Sensor sensor, Stack<Observation> obsStack) {
		this.connection = connection;
		this.obsStack = obsStack;
		this.sensor = sensor;
	}

	@Override
	public void run() {
		tracking = true;
		// register sensor
		connection.registerSensor(sensor);
		int counter = 0;
		long bytes = 0;
		long latency = 0;
		while (tracking || !obsStack.empty()) {
			// wait 200 millisecond before checking the observationstack
			try {
				Thread.sleep(200);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			if (!obsStack.empty()) {
				// get first observation out of stack
				Observation obs = obsStack.peek();
				// insert observation
				boolean insertSuccessful = connection.insertObservation(obs);
				if (insertSuccessful) {
					long latencyTemp = System.currentTimeMillis() - obs.getLatencyTime();
					latency += latencyTemp;
					Logging.addEntry("Latency for Obs: " + latencyTemp);
					obsStack.remove(obs);
					counter += 1;
				}
			}
		}
		connection.stop();
		Logging.addEntry(counter + " Obs in SOS");
		Logging.addEntry("Need " + bytes + " bytes for Communication");
		if (counter != 0) {
			Logging.addEntry((bytes / counter) + " Bytes per Obs");
		}
		Logging.addEntry("Average latency: " + (latency/counter));
	}

	public void setTracking(boolean tracking) {
		this.tracking = tracking;
	}

}
